@@ -16,14 +16,23 @@
 */
 
 /*SH_doc
-@NAME ped - string edit api using peg. allows binary input.
-@SYNOPSIS
-<pre>
+title=ped section=3 repnl=\040
+@name	ped
+@_brief	string edit api using peg. allows binary input.
+@_syno
+#include "ped.h"
+
+ped_t*	ped_new(const char* mode, const char* rstr
+	[, int rstrsz, const char* nlstr, int nlstrsz]);
+void	ped_free(ped_t* obj);
+ped_rt	ped_parse(ped_t* obj, const char* instr [, int instrsz]);
+void	ped_reset(ped_t* obj);
+char*	ped_fileread(const char* name [,int* sz]);
+
 typedef struct pedstate_tag{
-	const char* ruleinfo;
+	const char* info;
 	const char* emsg;
 	// other opaque members
-	// ...
 } ped_t;
 
 typedef struct ped_rtntag{
@@ -33,79 +42,142 @@ typedef struct ped_rtntag{
 	int binsz;
 } ped_rt;
 
-ped_t* ped_new(const char* mode, const char* rstr [, int rstrsz, const char* nlstr, int nlstrsz]);
-void ped_free(ped_t* obj);
-ped_rt ped_parse(ped_t* obj, const char* instr [, int instrsz]);
-void ped_reset(ped_t* obj);
-char* ped_fileread(const char* name [,int* sz]);	//fread() wapper
-</pre>
+@tl;dr
+	@(code)
+ #include "ped.h"
+ #include <stdio.h>
+ int main(){
+ 	const char* rstr = "R<-[a-z] {_0='@' _0}";  //ab12 >> @a@b12
+ 	ped_t* ped = ped_new("r", rstr);
+  	ped_rt res = ped_parse(ped, "ab12");
+ 	while(1){
+ 		if(res.rc == 0){ break; }	//complete
+ 		else if(res.rc == -1 || res.rc>0){ res=ped_parse(ped, NULL);}
+ 		else if(res.rc == -2){
+ 			printf("%.*s", res.binsz, res.bin );  //result
+ 			res = ped_parse(ped, NULL);	//NULL: send EOF
+ 		}
+ 		else if(res.rc== -10){
+ 			printf("catch ERR: %s\n", res.emsg);
+ 			break;
+ 		}
+ 	}
+ 	ped_free(ped);
+ }
+ //~$ cc src.c libped.a -lm -ldl
+	@()@
 
-@DESCRIPTION
-see example or ~$ ped -H for detail usage.	
+@_desc
+	see ~$ ped -H or exsample for ped basic usage. --
+	'ped_new()' makes parser. 'ped_free()' destroies parser.
+	'ped_parse()' reads string and return result.
+	'ped_reset()' set init the parser. --
+	--
+	'FUNC'
+	@(code)
+	ped_t* ped_new(mode, rstr [, rstrsz, nlstr, nlstrsz]);
+		eg) pobj = ped_new("nr", "R <- [a-z] {_0='@'}");
+		eg) pobj = ped_new("nr", "R <- [a-z] ", -1, "\r\n", 2);
+	@()--
+	
+	'PARAM'
+	@(list)
+	'	''_mode': parser setting. mode="dr" works as ~$ ped -d -r
+	@(code)@
+	  r: use expand mode
+	  R: use expand mode with locale 'C' regex
+	  n: noout if hit norule. same as ~$ sed -ne '/xyz/'
+	  N: stop if hit norule.
+	  t: output CST string instead of normal result.
+	  T: output full CST string. slow.
+	  d: set pegrule infostr to rtn->info.
+	@()--
+	below opts dont make parser. you cant use ped_parse().
+	@(code)@
+	  h: set usage str to rtn->info.
+	  H: set detail usage str to rtn->info.
+	  V: set ped application version msg to rtn->info.
+	@()--
 
-<pre>
-- ped_t* ped_new(mode, rstr [, rstrsz, nlstr, nlstrsz] ): make parser
+	'	''_rstr'		: ''pegrule string ptr. eg) rstr="rule1 <- 'abc'? [0-9]"''
+	'	''_[rstrsz]'	: use strlen(rstr) if -1/noset.
+	'	''_[nlstr]'	: just only uses for emsg info. dfl is "\n". string is
+		convert with c99 charesc parser as eval. "\n"="\\n"="\134n"="\\u000a".
+		this arg never affects parsing result. allow any str. eg) "x\0z"
+	'	''_[nlstrsz]'	: use strlen(nlstr) if -1/noset.
+	@()--
+	
+	'RETURN': malloc()ed ptr. you always have to ped_free().	--
+	@(list)
+	'	'rtn->'info': set infostr if you use 'd,h,H,V' opt.
+	'	'rtn->'emsg': set emsgstr and errno if pegrule is invalid.
+	'	'rtn->(other) : obj holds other opaque members.
+	@()--
 
-  mode: option setting string. mode="dr" works as ~$ ped -d -r
-    r: use expand mode
-    R: use expand mode with locale 'C' regex
-    n: noout if hit norule. same as ~$ sed -ne '/xyz/'
-    N: stop if hit norule.
-    t: output CST string instead of normal result.
-    T: output full CST string. slow.
-    d: set pegrule infostr to rtn->ruleinfo.
-    
-    -- below 2 opts doesnt make parser. you cant use ped_parse().
-    h: set usage str to rtn->ruleinfo.
-    H: set detail usage str to rtn->ruleinfo.
-    V: set ped application version msg to rtn->ruleinfo.
-  
-  rstr  : pegrule string ptr. eg) "rule1 <- 'abc'? [0-9]"
-  rstrsz: optional. use strlen(rstr) if noset.
-  nlstr : optional. just only use for emsg info. dfl is "\n". string is
-         convert with c99 charesc parser. "\n"="\\n"="\134n"="\\u000a".
-         this arg never affects parsing result. allow any str. eg) "x\0z"
-  nlstrsz: optional. use strlen(nlstr) if noset.
-
-  return: ped_t* rtn, malloc()ed ptr. you always have to ped_free().
-    rtn->ruleinfo: set infostr if you use 'd,h,H,V' opt.
-    rtn->emsg    : set emsgstr and errno if pegrule is invalid.
-  
-- ped_free(obj): destroy obj
-
-- ped_parse(obj, instr [, instrsz]): i/o api. substitute stdin/stdout
-  obj    : ped_new() return ptr.
-  instr  : input strptr. send EOF if set NULL. accept len 0 str "". 
-  instrsz: optional. use strlen(instr) if noset. send EOF if set -1.
-
-  return: ped_rt rtn, rtn.rc holds rtndata type.
-   rtn.rc>0: request str. rc=8 means 'send 8 or more byte if you can'
-     0 : parsed all input. completely suceed.
-    -1 : request all string until EOF.
-    -2 : rtn parsed str. see rtn.bin. set ped_parse(obj,"") to continue
-    -10: catch err. see rtn.emsg. state was reset/initilized.
-    ...if rc==7 but leftstr/buffsz is only 3byte, send 3byte plz.
-  
-  rtn.bin  : set result str if rtn.rc= -2. ptr exists in inner buff.
-  rtn.binsz: str size.
-  rtn.emsg : set emsg str if rtn.rc= -10. ptr exists in inner buff.
-  ...inner buffer is short-lived, so save it if necessary.
-
-- ped_reset(obj): reset state. prepare for parse other file/input etc
+	--
+	'FUNC'
+	@(code)
+	void ped_free(obj);
+	//destory the parser
+	@()--
  
-- ped_fileread(name [,int* sz]): file reader. fread() wrapper
-  name: filename. eg) ped_fileread("myrule.peg")
-  sz: optional. set bytesize if exists. eg) ped_fileread("a.txt", &buf)
+	--
+	'FUNC'
+	@(code)
+	ped_rt ped_parse(obj, instr [, instrsz]);
+	//push type parsing. use as substitute stdin/stdout.
+	@()--
+	
+	'PARAM'
+	@(list)
+	'	''_obj'	: ped_new() return ptr.
+	'	''_instr'	: input strptr. send EOF if set NULL
+	'	''_[instrsz]': use strlen(instr) if noset. send EOF if set -1.
+	@()--
+	
+	'RETURN': parse result data/emsg etc.
+	@(list)
+	'	'rtn.'rc': answer from parser.
+	@(code)@
+	>0 : request str. rc=8 means 'send 8 or more byte if you can'
+	 0 : parsed all input. completely succeed.
+	-1 : request all string until EOF.
+	-2 : rtn parsed str to rtn.bin. do ped_parse(obj,"") to continue
+	-10: catch err. see rtn.emsg. state was reset/initilized.
+	...if rc==7 but leftstr/buffsz is only 3byte, send 3byte plz.
+	@()--
+	'	'rtn.'bin'  : set parse result if rtn.rc= -2. ptr exists in inner buffer.
+		inner buffer is short-lived, so save it if necessary.
+	'	'rtn.'binsz' : bite size. retn.bin may holds \0
+	'	'rtn.'emsg' : set emsg str if rtn.rc= -10. ptr exists in inner buff.
+	@()--
 
-  return:char*, malloc()ed bin/emsg(errno!=0). you always have to free()
-</pre>
+	'FUNC'
+	@(code)
+	void ped_reset(obj);
+	//reset state. prepare for parse other file/input etc
+	@()--
+	--
+	'FUNC'
+	@(code)
+	char* ped_fileread(name [,int* sz]);
+	//file reader. fread() wrapper.
+	@()--
+	--
+	'PARAM'
+	@(list)
+	'	''_name'	: filename. "abc.txt" etc.
+	'	''_[*sz]': write file bytesize if exists. --
+		eg) s = ped_fileread("a.txt", &sz);	
+	@()--
+	
+	'RETURN': malloc()ed bin/emsg(errno!=0). you always have to free()
 
-@EXSAMPLE
-```
+@_eg
+	@(code)
 #include <stdio.h>
 #include "ped.h"
 int main(int argc, char** argv) {
-	// see detail for pegrule: ~$ ped -H
 	const char* rstr =
 		" RULE1 <- 'abc'	{_1 = 'ABC'} "
 		" RULE2 <- [0-9]	{_1 = '@' _1 '@ ' } "
@@ -118,7 +190,7 @@ int main(int argc, char** argv) {
 	ped_rt res = ped_parse(obj, s);
 	while(1){
 		if(res.rc == 0){ break; }
-		else if(res.rc == -1 || res.rc>0){ res = ped_parse(obj, NULL); }
+		else if(res.rc == -1 || res.rc>0){ res=ped_parse(obj, NULL); }
 		else if(res.rc == -2){
 			printf("%.*s", res.binsz, res.bin );
 			res = ped_parse(obj, NULL);
@@ -131,7 +203,7 @@ int main(int argc, char** argv) {
 // ~$ gcc src.c libped.a -lm -ldl
 // ~$ ./a.out	#>> ruleinfo + ABC***@1@ @2@ @3@ *
 
---- 
+//--- 
 #include <stdio.h>
 #include <stdlib.h>
 #include "ped.h"
@@ -159,23 +231,21 @@ int main(int argc, char** argv) {
 }
 // ~$ gcc -static -Wall -pedantic src.c libped.a -lm -ldl
 // ~$ echo "if(a==0){a=1}" | ./a.out	#>> if a==0 then a=1 end
-```
+	@()
 
-@OPTIONS	-
-@EXIT_STATUS ped_new(), ped_fileread() set errno if err/failed.
 @RETURN_VALUE
-<pre>
+	@(code)
 ped_t* ped_new()  :	suc/fail == ptr/ptr(errno!=0, ptr->emsg)
 void ped_free()   : -
 ped_rt ped_parse(): check rtn.rc. set rtn.emsg if rtn.rc= -10(err)
 void ped_reset()  : -
-char* ped_fileread(): suc/fail == ptr/ptr(errno!=0)
-</pre>
-@ERRORS output emsg and exit(1) if fatal err.
-@NOTES
+char* ped_fileread(): suc/fail == ptr/ptr(errno!=0, emsg)
+	@()
+@ERRORS output emsg to stderr and exit(1) if fatal err.
+@notes
+	@(code)
 - sloppy benchmark:
 
-<pre>
 -- ped vs sed	(1cpu 2.8GHz)
 sed  : sed -e 's@[a-zA-Z_][a-zA-Z0-9_]*@-@g'
 ped  : ped -re 'rule1 <- ![0-9] [a-zA-Z0-9_]+	{_0 = "-"}'
@@ -183,17 +253,16 @@ ped  : ped -re 'rule1 <- ![0-9] [a-zA-Z0-9_]+	{_0 = "-"}'
 	sed: real 0m0.517s
 	ped: real 0m0.618s
 	...130-150ms to convert 1000 lines (in ped self-hosting) 
-</pre>
-
+	@()
 @CONFORMING_TO POSIX.1-2001+
-@BUGS \-
 @COPYRIGHT Copyright 2020 momi-g, GPLv3+
-@VERSION 2021-07-10 v2.0.1
+@VERSION 2021-08-15 v2.1.0
 @SEE_ALSO
-<pre>
+	@(pre)
+~$ ped -H
 https://pdos.csail.mit.edu/papers/parsing:popl04.pdf
 https://en.wikipedia.org/wiki/Parsing_expression_grammar
-</pre>
+	@()
 //SH_docE*/
 //set_clists()でgnuregのfastmapが必要
 #ifndef _GNU_SOURCE
@@ -274,6 +343,7 @@ typedef struct msgserver_tag {
 	int vflg;
 	char* emsg;	//jstとややかぶるけど元締めとして保持する必要がある
 	int seof;
+	int sbof;
 	int bcnt;
 	int nlcnt;
 	int ccnt;
@@ -368,10 +438,12 @@ typedef struct peg_jumpstate_tag {
 	sg_t* instr;
 	sg_t* outstr;
 	int curpos;	//now charpos
+	int pre_curpos;	//detect loop
 //
 	int rpos;
 	int tpos;
 	int seof;
+	int sbof;
 	resarr_t* resarr;
 	int resepos;
 	int resmaxsz;
@@ -394,7 +466,7 @@ typedef struct peg_editor_buff_tag {
 
 // fulldata
 typedef struct pedstate_tag {
-	const char* ruleinfo;
+	const char* info;
 	const char* emsg;
 //
 	msv_t* msv;		//msg_server
@@ -517,12 +589,13 @@ ped_t* ped_new_impl(const char* mode, const char* rstr, int rstrsz, const char*
 	ped_t* ped = (ped_t*)calloc(1, sizeof(ped_t) );
 	;STOP(ped==NULL, "calloc() failed");
 //info
-	if( strchr(mode,'h') ){ ped->ruleinfo = ped_info("h", rstr, rstrsz); }
-	else if( strchr(mode,'H') ){ ped->ruleinfo = ped_info("H", rstr, rstrsz); }
-	else if( strchr(mode,'V') ){ ped->ruleinfo = ped_info("V", rstr, rstrsz); }
-	else if( strchr(mode,'d') ){ ped->ruleinfo = ped_info(mode, rstr, rstrsz);}
+	if( strchr(mode,'h') ){ ped->info = ped_info("h", rstr, rstrsz); }
+	else if( strchr(mode,'H') ){ ped->info = ped_info("H", rstr, rstrsz); }
+	else if( strchr(mode,'V') ){ ped->info = ped_info("V", rstr, rstrsz); }
+	else if( strchr(mode,'d') ){ ped->info = ped_info(mode, rstr, rstrsz);}
 	if( strspn(mode, "hHV") ){ return ped; }
 //maindata
+
 	ped->msv = msv_new(mode, nlstr, nlstrsz);
 //	
 	errno=0;
@@ -535,19 +608,21 @@ ped_t* ped_new_impl(const char* mode, const char* rstr, int rstrsz, const char*
 	}
 	//情報追加
 	if( strchr(mode,'d') ){
-		char* s = (char*)ped->ruleinfo;
+		char* s = (char*)ped->info;
 		char* ss = clists_info(ped);
 		int len = strlen(s)+ strlen(ss);
 		char* p = malloc(len+20);
 		sprintf(p, "%s\n--fastmap_info\n%s", s, ss);
 		free(s);
 		free(ss);
-		ped->ruleinfo = p;
+		ped->info = p;
 	}
 //
+	errno=0;
 	ped->jst = jst_new(ped->rdata->nlstr, ped->rdata->nlstrsz);
 	if(errno){ emsg=(char*)ped->jst; goto lb_ERR; }
 	ped->edbuf = edbuf_new();
+
 //
 //launch subproc: fknum, ppid, fork()
 	int fknum = ped->msv->jobs -1;
@@ -583,7 +658,7 @@ void ped_free_impl(ped_t* ped){
 	if(ped==NULL){return;}
 //	reset_jobs(ped, -1);	//	kill fork
 // forkの前の可能性もあるのでreset系は使わずshutdown 運用でreadになってるはず
-	free((void*)ped->ruleinfo);		//const外し
+	free((void*)ped->info);		//const外し
 	free((void*)ped->emsg);
 //
 	if(ped->msv && ped->msv->jobs>1){
@@ -615,6 +690,7 @@ void ped_reset(ped_t* ped) {
 	msv->nlcnt=0;
 	msv->ccnt=0;
 	msv->seof=0;
+	msv->sbof=0;
 	msv->wait_jidxinfo=0;
 //	
 	idq_reset(msv->strq);
@@ -787,6 +863,7 @@ static msv_t* msv_new(const char* mode, const char* nlstr, int nlstrsz){
 	if(p){ jobs = (int)strtol(p, NULL, 10); }
 	if(jobs<=0){jobs=1;}
 	msv->jobs = jobs;
+	errno=0;
 	msv->sfdarr = (int*)calloc(jobs, sizeof(int));	//selfの分 sentinelは止めた
 	; if(errno){ return NULL; }
 	msv->pidarr = (int*)calloc(jobs, sizeof(int));
@@ -855,7 +932,7 @@ static rdata_t* rdata_new(const char* mode, const char* rstr, int rstrsz, const
 	obj->nlstr = res.s;
 	obj->nlstrsz = res.sz;
 //dbg(obj->nlstr, obj->nlstrsz);
-	lbc_directf(lvm,
+	rc = lbc_pdirectf(lvm,
 		" local rstr = " LBC_P2S
 		" local mode = " LBC_P2S
 		" local m = require('ped_ljmod')"
@@ -863,6 +940,7 @@ static rdata_t* rdata_new(const char* mode, const char* rstr, int rstrsz, const
 		" return gsave"
 	, rstr, rstrsz, mode, strlen(mode) );
 	res = lbc_tbval(lvm, 0, 1);
+	if(rc<0){ emsg = strndup(res.s, res.sz+1); goto lb_ERR; }	//assert
 	if(res.tp=='N'){
 		//nil...err pegerr nstr is '\n'. parseerr nstr os selectable
 		res = lbc_tbval(lvm, 0, 2);
@@ -892,7 +970,7 @@ static rdata_t* rdata_new(const char* mode, const char* rstr, int rstrsz, const
 	rpos=0;
 	for(; obj->rhead[rpos]>=0;rpos++);;
 	obj->norule_rpos = rpos;
-//	
+//
 	return obj;
 lb_ERR:;
 	rdata_free(obj);
@@ -914,6 +992,7 @@ static void rdata_free_impl(rdata_t* rdata) {
 static char* set_rglists(rdata_t* rdata) {
 //idxはluaに合わせる
 	char* emsg=NULL;
+	errno=0;
 	rdata->svlocale = strdup( setlocale(LC_CTYPE, NULL) ); //localeはcloseまで永続
 	; if(errno){return strdup( strerror(errno));}
 //0は無いはず 2は"C" 固定でいく
@@ -941,7 +1020,10 @@ static char* set_rglists(rdata_t* rdata) {
 	dq_t* dq = pdq_new();
 	for(int i=0; rdata->rtokens[i].num!= -9; i++) {
 		const char* ctg = rdata->rtokens[i].ctg;
-		if(strcmp(ctg, "reg")!=0) { pdq_push(dq, NULL); continue;}
+		int syn = 0;
+		if( strcmp(ctg, "bre")==0){ syn = ureg_bre_syn(); }
+		else if( strcmp(ctg, "ere")==0){ syn = ureg_ere_syn(); }
+		if(syn==0) { pdq_push(dq, NULL); continue;}
 		ureg_t* robj=NULL;
 		size_t rc= (size_t)-1;
 		char* sp = (char*)rdata->rtokens[i].data;
@@ -970,7 +1052,7 @@ static char* set_rglists(rdata_t* rdata) {
 				dp++;
 			}
 		}
-		robj = ureg_new_raw(mm, dp-mm);	//m ,sz
+		robj = ureg_new_raw(mm, dp-mm, syn);	//m ,sz
 		if(robj==NULL){
 			int rpos=0;
 			for(;;rpos++){if(i< rdata->rhead[rpos]-1){break;} }
@@ -1015,6 +1097,7 @@ static void reset_rglists(rdata_t* rdata) {
 
 static char* set_clists(rdata_t* rdata) {
 	char* emsg=NULL;
+	char emsg_buf[256]={0};
 	/*
 	ID <- "123"	...1
 	ID <- "123"* "xyz"	...1,x
@@ -1031,7 +1114,11 @@ static char* set_clists(rdata_t* rdata) {
 		//戦略は99%のruleは失敗で1%は成功だから一文字で多くの失敗を弾ける
 		//疑陽性も出るが80%ぐらいは削減 0は確実に失敗 -1は確実に未消費成功
 	*/
-	// !系は 判定が複雑な割には効率が悪いのでスキップ classだけは使える
+	// 方針は1stヒットで進行する可能性があるchar達を集める
+	// arrのフラグとしては0, 1が正規兵, -1が予備 -2はjmp検査結果のNGワードで
+	// 絶対にloopを引き起こす奴 最後にmergeするときに作成する
+	
+	// !系は 判定が複雑で未消費だしスキップ classだけは使える
 	// *?系は勘定に入るが後続も続く 無しならそこでstop 最後まで行ったら-1
 	// jmpは再起jmpで葉まで探索してから遡上する 合成時はnextもチェックする
 	// !''*+系は弾いてるので存在しない >> 疑陽性が出るので実行時もloop検査
@@ -1044,7 +1131,7 @@ static char* set_clists(rdata_t* rdata) {
 	//typedef char (*fm_t)[256];	//fastmap
 	typedef char fm_t[256];
 	fm_t* clists = (fm_t*)calloc(256*erpos, sizeof(char) );
-	char* lprpos = (char*)calloc(erpos, sizeof(char) );	//loop
+	char* lprpos = (char*)calloc(erpos, sizeof(char) );	//loop, 内部利用だけのフラグメモ
 	char* fillrpos = (char*)calloc(erpos, sizeof(char) );	//carrが完了したruleflg
 	dq_t* rstk = idq_new();	//identで飛ぶ度にpush. orを見るときはスルー
 	dq_t* tstk = idq_new();
@@ -1057,7 +1144,12 @@ static char* set_clists(rdata_t* rdata) {
 lb_JMP:;
 			char abuf[256]={0};
 			rtoken_t tk = rdata->rtokens[idx+tpos];
-			// !系はlit/reg/identで意味を持たない。![]だけは手がかりにつかえる
+			// !はlit/reg/identで意味を持たない。![class]は手がかりにつかえる
+			// &はそのままフィルタで使える
+			// identの clist -2はpruneのかなり重要な部分。永久ループを検出
+			// するのに実行時に毎回検査してたらovhがバカでかくなる。
+			// 検査ロジックも複雑
+			// 事前にいけない奴を纏めておくとO(1)ですむので速い
 			if(tk.ctg[0]=='i') {
 				if(tk.pre[0]=='!'){	continue; }
 				//pass系は0が可能性が残ってる. 最後までいったら-1フラグ				
@@ -1066,46 +1158,47 @@ lb_JMP:;
 				//下準備が終わってるなら追加してく
 				while( fillrpos[dst] ){
 					for(int i=0;i<256;i++){ if(clists[dst][i]==1){clists[rpos][i]=1; } }
-					dst++;	//共通で使うので先にdef
+					dst++;	//共通で使うrnumなので先にdef
 					const char* p="";
 					if(rdata->rhead[dst]>0) { p=rdata->rtokens[rdata->rhead[dst] -1].data; }
 					if( fillrpos[dst-1]== -1 ){
+						//bofとeofがひっかかる。全体で見れば消費系なので
+						//luaで0消費じゃなくて1消費にして逃がすか。
+						//唯一品だから扱いが面倒だ
 						//本来のjmp先 上で++してるので--辻褄合わせ
 						//未消費系は伝染する
 						fillrpos[rpos]= -1;
-						if(strcmp(dstname,p)==0 ){
-						//未消費系なのに後ろが居る 絶対に到達しないerr
-				char sbuf[256]={0};
-				sprintf(sbuf
-				, "ERR: rule %s:%d,%d: unreachable rule. use cmtout '#'"
-				, p, dst, 0);
-							emsg=strdup(sbuf);
-							free(clists);
-							clists=NULL;
-							goto lb_RTN;			
-						}
+//						if(strcmp(dstname,p)==0 ){
+//						//未消費系なのに後ろが居る 絶対に到達しないerr
+//						// !&のフィルタ系rule系なら全hit。弾くのはダメだ
+//				sprintf(emsg_buf
+//				, "ERR: rule %s:%d,%d: unreachable rule. use cmtout '#'"
+//				, p, dst+1, 0);
+//							goto lb_ERR;			
+//						}
 						//未消費系はnext
 						tpos++;
 						goto lb_JMP;
 					}
-					//消費系でも後ろがあるなら追加する
+					//消費系でも後続rnumがあるなら追加する
 					if(strcmp(dstname,p)==0 && fillrpos[dst] ){ continue; }
 					if(strcmp(dstname,p)==0 ){
 						//同名だけど未挑戦 >>見に行って戻る
 						//重複になるけどloopで潰す
 						idq_push(rstk, rpos);
 						idq_push(tstk, tpos);
-						//lprpos[rpos]=1;	//countは初回にやってるので不要
+						//lprpos[rpos]=1;	//countは初回jmp先でやる
 						rpos=dst;
 						tpos=0+1;	//+tpos_headはforがある
 						idx = rdata->rhead[rpos] -1;	//lua -1 
 						goto lb_JMP;
 					}
-					//全員消費系で完了 >> *があるかもしれない
-					fillrpos[rpos]= 1;
+					//全員調査完了で本ルート >> *があるかもしれない
+					//中間で未消費成功がいるなら伝染する *+系のあぶり出しに必須
+					if(fillrpos[rpos]==0){ fillrpos[rpos]= 1;}
 					goto lb_WALK;
 				}
-				//初回系
+				//jmp先が初訪問
 				if( lprpos[tk.num-1] ==0 ){
 					//初物 現在位置メモ
 					idq_push(rstk, rpos);
@@ -1117,6 +1210,7 @@ lb_JMP:;
 					continue;
 				}
 				//以下loop検知** lprposから関連clistが必須系になる
+				// 途中で+?系があったりする危なっかしいruleだけど合法
 				//先頭char達以外だとloopしてしまうため後続は関係なくなる
 				//再帰下降パーサの仕組みからしてloop脱出のメモ参照脱出は違法か
 				//()のネストとかは再帰で処理できるのでセマンティクスが違っちゃう
@@ -1134,12 +1228,12 @@ lb_JMP:;
 				// 1とは別に-2を入れて自動エラー発生させるか inputがおかしいとか
 				
 				//1以外の先頭入力は即死エラー -2
-				for(int i=0; i<256; i++){ abuf[i]!=1 ? abuf[i]= -2: 0; }
+				for(int i=0; i<256; i++){ abuf[i]!=1 ? abuf[i]= -2: 1; }
 				//全員に配布 fillも埋める
 				for(int i=0; i<idq_len(rstk); i++){
 					int rnum = idq_see(rstk, i);
 					memcpy(clists[rnum], abuf, 256);
-					fillrpos[rnum] = -2;	//消費しないとloopなので-2
+					fillrpos[rnum] = -2;	//消費しないとloopなので-2 実行時検査に必要
 				}
 				//reset
 				rpos = idq_see(rstk, 0);
@@ -1153,49 +1247,58 @@ lb_JMP:;
 				break;
 				//goto lb_WALK;
 				//identとしては消費系だけど*?がついているかもしれない
-				//完了してるのでbreakです
+				//再起調査が完了してるのでbreakで通常コースに戻る
 			}
 			else if(tk.ctg[0]=='c'||tk.ctg[0]=='a') {
 				memcpy(abuf, tk.data, 256);
-				//classのみ!系があっても1byteなのでok
-				if(tk.pre[0]=='!'){ CBIT_NOT(abuf, 256);}
+				if(tk.pre[0]=='!'){ CBIT_NOT(abuf, 256);} //classのみ!系利用可
 				for(int i=0;i<256;i++){
 					if(clists[rpos][i]!=1){ clists[rpos][i] = !!abuf[i]; }
 				}
 			}
 			else if(tk.ctg[0]=='l') {
-				if(tk.pre[0]=='!'&&tk.num!=0){ continue; }	// !系は分からない
+				// loop不正の温床なのでチェックする ''+?*系
 				; STOP(tk.suf[0]&&tk.pre[0], "fatal, pre+suf detect");
-				if(tk.pre[0]=='!'&&tk.num==0){
-					// !''は完全失敗。!&と*+?はloopckで弾いてるので後続はない
-					memset(clists[rpos], 0, 256);
-					; STOP(tk.suf[0], "fatal, !''+ detect?");
+				if(tk.suf[0]!='\0'&&tk.num==0){
+					const char* rname = rdata->rtokens[idx].data;
+					sprintf(emsg_buf
+					, "ERR: rule %s:%d,%d: ''?+* is redundant/loop grammar"
+					, rname, rpos, tpos);
+					goto lb_ERR;
 				}
-				//無条件成功なので自動的に次に回す
-				else if(tk.num==0){ continue; }
+				if(tk.pre[0]=='!'||tk.num==0){ continue; }	// !系は分からない
+			//	if(tk.pre[0]=='!'&&tk.num==0) 完失系だけど始末が大変 実行時判定
 				else {
-					//フツーのlit系。+*が居るかも
+					//フツーのlit系。+*?が居るかも 共通で追加判定する
 					//tk.dataはchar*でマイナスになるのでcastが必要 ハマった
 					int c = (unsigned char)tk.data[0];
 					clists[rpos][c]=1;
 				}
 			}
-			else if(tk.ctg[0]=='r') {
+			else if(tk.ctg[1]=='r') {
 				if(tk.pre[0]=='!'){	continue; }
-				//rglistはrtokensの羅列idxをNULL付きで一列にrgdataを入れてる 殆ど空だけど
+				//rglistはrtokensの羅列idxをNULL付きで一列に入れてる 殆ど空
 				char* fmap = rdata->rglists[idx+tpos]->fastmap;
-				// ''系も壁も居るが検査
-				int i=0; for(;i<256&&fmap[i]==0;i++);;	if(i==256){i= -1;}	//壁判定
-				//絶対失敗判定
-				if(i<0 && tk.suf[0]!='*' && tk.suf[0]!='?'){ memset(clists[rpos], 0, 256); }
-				else {
-					//next有りのフツーのadd処理 !は最初に弾いてるので*?+が
-					// sucfailあるけどガバガバでも処理量が増えるだけで合法
-					for(int i=0;i<256;i++){
-						if(clists[rpos][i]!=1){ clists[rpos][i] = !!fmap[i]; }
-					}
+				// ''系も壁も居るが実行時に回す fmで1と0にやってくれてる
+				// //と/./は区別不可能
+				for(int i=0;i<256;i++){
+					if(clists[rpos][i]!=1){ clists[rpos][i] = !!fmap[i]; }
 				}
 			}
+			else if(tk.ctg[0]=='e') {
+				//eofの分類としては''と同じ未消費系で全て失敗なので
+				//clistは許可系候補リストなので何もしない
+				if(tk.pre[0]=='!'){	continue; }
+			}
+			else if(tk.ctg[0]=='b') {
+				//eofの分類としては''と同じ未消費系で全て失敗なので
+				//clistは許可系候補リストなので何もしない
+				if(tk.pre[0]=='!'){	continue; }
+				//beginは常に成功する。後続文字があろうとなかろうと関係ない
+				// ^$は入力無しでも成立するのと同じ
+				memset(clists[rpos], 1, 256);
+			}
+			//bof追加 無条件pass 深判定はjmpでcurpos使って行う
 lb_WALK:;			
 			//walk判定 *? なら追加有り +&系は完了 ! ""は個別に始末済
 			if(tk.suf[0]=='*'||tk.suf[0]=='?') {continue;}
@@ -1206,7 +1309,7 @@ lb_WALK:;
 				for(int i=0;i<256;i++){	clists[rpos][i]=(clists[rpos][i]||-1); }
 				fillrpos[rpos] = -1;	//carrが完了したruleflg -1は未消費1は消費
 			}
-			//消費フラグ eofは直接ここに流れ着く
+			//調査完了フラグ
 			if(fillrpos[rpos]==0){ fillrpos[rpos]=1; }
 
 			//jmpがあれば戻るし
@@ -1217,14 +1320,16 @@ lb_WALK:;
 				idx = rdata->rhead[rpos] -1;	//lua -1
 				continue;
 			}
-			//なければおしまい
+			//なければ区切りで次のruleへ
 			break;
 		}
 	}
 	
-	//pass_2: 隠れ!ident*を検査
+	//pass_2: 隠れ!ident(*)を検査 rgの関係でここでしか出来ない
+	// 実行時検査が最終防衛ラインでloopを弾くけどclistとしては
+	// 守備範囲で必要だろう
 	for(int rpos=0; rpos<erpos; rpos++) {
-		;STOP(fillrpos[rpos]==0, "fillrpos leak" );
+		;STOP(fillrpos[rpos]==0, "fatal, fillrpos leak" );
 		int idx = rdata->rhead[rpos] -1;	//lua分 rpos0はheader
 		int cur=1; 	//1スタート 0はheadが入る
 		for(;;cur++) {
@@ -1232,17 +1337,15 @@ lb_WALK:;
 			if(tk.ctg[0]=='s') {break;}
 			if(tk.pre[0]&&tk.ctg[0]=='i'&&fillrpos[tk.num-1]<0){ cur= -cur; }
 			//存在しないはず 事前に弾いているはず
-			else if(tk.pre[0]&&tk.suf[0]){ cur= -cur; STOP(1, "miss" );}
+			//ruleloopはluaで検査してるけど、*の未消費は実行時検査でないと
+			//誤爆する。ここでは!*の検出のみ identが隠れ*になってる場合を報告
+			else if(tk.pre[0]&&tk.suf[0]){ cur= -cur; STOP(1, "fatal err: !& *+?");}
 			if(cur<0){
-				char sbuf[256]={0};
-				sprintf(sbuf, "ERR: rule %s:%d,%d: \n"
-					"detect illigal '&!rule', no char consume grammar == !__*\n"
+				sprintf(emsg_buf, "ERR: rule %s:%d,%d: %s\n"
+					"detect illigal rule jump, same as !&rule == !&__*\n"
 					"&__* / !__?: always suc/fail"				
-					,rdata->rtokens[idx].data, idx+1, -cur);
-				emsg=strdup(sbuf);
-				free(clists);
-				clists=NULL;
-				goto lb_RTN;
+					,rdata->rtokens[idx].data, idx+1, -cur, tk.data);
+				goto lb_ERR;
 			}
 		}
 	}
@@ -1254,6 +1357,11 @@ lb_RTN:;
 	idq_free(tstk);
 	rdata->clists=clists;
 	return emsg;	//noerr==NULL
+lb_ERR:;
+	emsg=strdup(emsg_buf);
+	free(clists);
+	clists=NULL;
+	goto lb_RTN;			
 }
 
 static void reset_clists(rdata_t* rdata) {
@@ -1308,6 +1416,7 @@ static char* clists_info(ped_t* ped){
 static jst_t* jst_new(const char* nlstr, int nlstrsz){
 	char* emsg=NULL;
 //
+	errno=0;
 	jst_t* res = (jst_t*)calloc(1, sizeof(jst_t) );		//ここでmemerrが起きる
 	; if(errno){ emsg=strerror(errno); goto lb_ERR; }
 	res->resarr = (resarr_t*)calloc( RESARR_INITSZ, sizeof(resarr_t) );
@@ -1340,7 +1449,8 @@ static void jst_free_impl(jst_t* jst) {
 static void jst_reset(jst_t* jst){
 	sg_clear(jst->instr);
 	sg_clear(jst->outstr);
-	jst->curpos=0;
+	jst->curpos= -1;
+	jst->pre_curpos= -1;	//なくても2回めで当たる>> bofの1stでやっぱいる
 	// result mem
 	jst->bcnt=0;
 	jst->nlcnt=0;
@@ -1351,7 +1461,6 @@ static void jst_reset(jst_t* jst){
 	jst->resepos = -1;
 	jst->ipos_c = -1;
 	jst->ipos_mem[0]= 0;
-
 	//ここでres_push()したいけどpedがないからうんこちゃん
 	//初回。情報追加しておく。curposとかidentとか重要マーカー情報があちこちで
 	//必要なのでvecセット
@@ -1467,7 +1576,7 @@ static void pfix_restore(ped_t* ped) {
 
 static int fix_result(ped_t* ped, int rc) {
 	//大変。suf/preの分類.tokenの体裁も整える rtn 0/1/2 loop, suc,fail,eof 色々
-	//clistのスキップも加わった 操作前に確定失敗なのでtopでいい
+	//clistによる絶対失敗-3, 調節済で即戻る
 	if(rc== -3){return -2;}
 	rtoken_t* tk = gettoken(ped, -1, -1);
 	jst_t* jst = ped->jst;
@@ -1475,13 +1584,21 @@ static int fix_result(ped_t* ped, int rc) {
 		// 成功でも未消費がありえる 未消費loopは散らばってて
 		// 厄介。accとここ。
 		// ここではident以外の単体系のみkickする
+		// ident系のjmpは再突入時に検査する
 		//単体系で成功ってことは文字が入ってないといけない
 		//loop系で eof*もありえるけど明らかに不正なのでkickでいい
+		// bof,eofも二回目以降に引っかかるだろう
 		//単体系はreseposに直前が露出してるので直接観測できる
-		if(tk->ctg[0]!='i' && jst->resarr[jst->resepos].curpos == jst->curpos){
-			jmp_seterr(ped, "detect 0 byte consume term");
+//		if(tk->ctg[0]!='i' && jst->resarr[jst->resepos].curpos == jst->curpos){
+//			jmp_seterr(ped, "detect 0 byte consume term");
+//			return -20;
+//		}
+//消費検査の方がスマートか 復活と検討
+		if(jst->curpos==jst->pre_curpos ) {
+			jmp_seterr(ped, "detect 0 byte consume");
 			return -20;
-		}
+		}		
+
 		res_push(ped->jst, TK_LOOP, 0);
 		return 0;
 	}
@@ -1491,10 +1608,6 @@ static int fix_result(ped_t* ped, int rc) {
 		rc= -1;
 	}
 	else if (rc== -2 && tk->suf[0]=='*') {
-		//resepos== -1もある 初期値で存在しない場合。完全勝利のとき
-		//>>初期resepos==0をMARKで追加して不要になった
-	//	int modeck = !TK_LOOP;
-	//	if(jst->resepos>=0){ modeck =jst->resarr[jst->resepos].mode; }
 		int modeck =jst->resarr[jst->resepos].mode;
 		if(modeck!=TK_LOOP) {
 			//*だが失敗即終了...lit/loop/loope下駄 loopなら二つ前がloopのはず
@@ -1506,13 +1619,10 @@ static int fix_result(ped_t* ped, int rc) {
 	}
 	else if (rc== -2 && tk->suf[0]=='+') {
 // '+' だけ失敗がある
-//		int modeck = !TK_LOOP;	//reseposは-1でidxエラーがある
-//		if(jst->resepos>=0){ modeck =jst->resarr[jst->resepos].mode; }
 		int modeck = jst->resarr[jst->resepos].mode;
 		if(modeck!=TK_LOOP) { rc= -2;}
 		else { res_push(ped->jst, TK_LOOPEND, 0); rc= -1; }
 	}
-
 // !&ならば共通巻き戻し
 	if(strlen(tk->pre)!=0) {
 		//res cur修正 &!共通	()系は%でRTNでrposが適切な位置になってるはず
@@ -1521,7 +1631,7 @@ static int fix_result(ped_t* ped, int rc) {
 		//反転なら操作
 		if(tk->pre[0]=='!' && rc==-1) { rc= -2; }	//成功を踏み潰す 
 		else if(tk->pre[0]=='!' && rc== -2) { rc= -1;  }
-		//ダミーが必要なら突っこむ restoreで全削除してるため
+		//ダミーが欲しいので突っこむ restoreで全削除してるため
 		if(rc== -1){ res_push(ped->jst, TK_LIT, 0); }
 	}
 	return rc;
@@ -1530,10 +1640,12 @@ static int fix_result(ped_t* ped, int rc) {
 // vicの度に飛んでくる ここで消費文字数を更新する
 // curposはvicならorderflush後にdelで0になるから単純加算でいい
 // 少し無駄だけど毎回全探索をかける。改良の余地有り
+#define curM(num)	((num==-1)?0:num)
 static void nlpos_update(ped_t* ped) {
 	jst_t* jst = ped->jst;
-	int cpos = jst->curpos;
-	if(jst->curpos<0){ cpos = -jst->curpos -1; }
+	int cpos = curM(jst->curpos);
+	//-1は成功時に全て0以上になるjmpfuncで入れてるのでeofだけ<0
+	if(jst->curpos<= -10){ cpos = -jst->curpos -10; }
 //
 	int nlcnt=0;
 	char* np = sg_ptr(jst->instr) + cpos;
@@ -1553,7 +1665,10 @@ static void nlpos_update(ped_t* ped) {
 	jst->ccnt = (int)(np - spp);	//累積しない
 }
 
-//idx==0はinitかMARKで初回位置を保持する 巻き戻しで必要
+//iposはstackの先頭位置のみの保持 !&や失敗の巻き戻りを高速化する
+// 失敗は殆どを占めるので大分効果的だと思われる idxはreseposのidx
+// でidentの時に積み上げてaccでpopする。常に先頭には直前のreseposが露出する
+//idx==0はinitかMARKで初回位置を保持する 巻き戻しで必要なので追加修正した
 //jmpしていればipos_cが1以上になる 0ならjmp無しでvicとかrootとか
 //初期代入は
 static void ipos_push(jst_t* jst, int idx) {
@@ -1575,22 +1690,21 @@ static int ipos_pop(jst_t* jst) {
 static const char* jmp_seterr(ped_t* ped, const char* msg){
 	int svrpos = ped->jst->rpos;
 	jst_t* jst = ped->jst;
-	const char* bin = sg_ptr(jst->instr)+jst->curpos;
-	
-	int presz = jst->curpos;
-	int pstsz = sg_sz(jst->instr)-jst->curpos;
+	const char* bin = sg_ptr(jst->instr)+curM(jst->curpos);
+	int presz = curM(jst->curpos);
+	int pstsz = sg_sz(jst->instr)-curM(jst->curpos);
 	if(presz>20){ presz=20; }
 	if(pstsz>20){ pstsz=20; }
 	
 	nlpos_update(ped);
-	sg_clear(ped->edbuf->sgbuf);
-	sg_addf(ped->edbuf->sgbuf
-	, "jmpERR: rule %d %s: l %d,%d(%d byte): %s: %.*s@%.*s"
-	, svrpos+1, gettoken(ped, svrpos, 0)->data, ped->msv->nlcnt + jst->nlcnt
+	sg_clear(ped->edbuf->sg1);
+	sg_addf(ped->edbuf->sg1
+	, "jmpERR: rule %d %s: l%d,%d(%d byte): %s: %.*s@%.*s"
+	, svrpos+1, gettoken(ped, svrpos, 0)->data, ped->msv->nlcnt + jst->nlcnt+1
 	, jst->ccnt, ped->msv->bcnt + jst->bcnt, msg
 	, presz, bin-presz, pstsz, bin
 	);
-	ped->jst->emsg = sg_ptr(ped->edbuf->sgbuf);
+	ped->jst->emsg = sg_ptr(ped->edbuf->sg1);
 	return ped->jst->emsg;
 }
 
@@ -1603,11 +1717,13 @@ static int jmp_suc(ped_t* ped) {
 	//acc / vic判定	drule作成時にendに-1のtokenを付加してる
 	if( gettoken(ped, -1, -1)->num<0 ) {
 // 戻り先を覗き見. curが進んでないのに+*系だったら永久になるので弾く
+// bofは-1から0になって自動失敗するから結果的に問題にならなそう
 //jmp直後にipos_push(,...resepos)してるので.curposが取り出せる
 //epos情報はaccもvicも共通で使うので先取り出来る？
 		int pos = ipos_pop(ped->jst);
 		resarr_t res = jst->resarr[pos];
 		if(res.curpos == jst->curpos){
+			//curM()は不要だと思う。成功ってことはresが入っていて0になるはず
 			rtoken_t* rtk = gettoken(ped, res.rpos, res.tpos);
 			if(rtk->suf[0]=='*'||rtk->suf[0]=='+'){
 				rc= -20;
@@ -1626,16 +1742,24 @@ static int jmp_suc(ped_t* ped) {
 			//vic
 			res_push(ped->jst, TK_MARK, -9);
 			ipos_push(ped->jst, jst->resepos);
-			//iposが-1になるのでcurpos目当てにstart位置入れ直し
+			//iposが-1になってるはずのでcurpos目当てにstart位置入れ直し
 			//rposはeditでcurrentのruleを探すのに使うのでrpos=0前に保存
 			//iposはcurのみチェックするのでrposがズレてても問題ない
-			//一致させておきたいところだけど
+			//一致させておきたいところだけど reseposとrposが仲良しなので諦める？
+			int svrpos=jst->rpos;
 			rc=0;
 			jst->rpos=0;
 			jst->tpos=0;
 			//量によって継続かeditか異なる
 			if(jst->curpos > ped->msv->bufsz_min ){ rc= -11; }	//報告なら-11, 継続は0
-			else if(jst->curpos<0) { rc=-10; jst->seof=2; }	//EOFヒット !. {_0 = "gw"} とか
+			else if(jst->curpos<= -10) { rc=-10; jst->seof=2; }	//EOFヒット !. {_0 = "gw"} とか
+			//致命傷 vicなのに無限ループしてる 2.0.0から復活
+			if(jst->curpos==jst->pre_curpos && jst->seof!=2) {
+				rc= -20;
+				jst->rpos = svrpos;
+				jmp_seterr(ped, "detect 0 byte consume");
+			}
+			jst->pre_curpos = jst->curpos;
 		}
 		goto lb_RTN;
 	}
@@ -1652,7 +1776,8 @@ static int jmp_fail(ped_t* ped) {
 	jst->tpos=0;
 	int grp = gettoken(ped, -1, -1)->num;
 	jst->resepos = jst->ipos_mem[jst->ipos_c];
-	//とりあえず共通巻き戻し popは失敗確定してから
+	//とりあえず共通巻き戻し nextruleもあるので popは失敗確定してから
+	//これが速いのでiposを記録する価値がある 高速が欲しいのでdqではなくarr
 	//jmp, next or fail
 	if(jst->ipos_c>0){
 		//jmp先で失敗した
@@ -1684,11 +1809,11 @@ static int jmp_fail(ped_t* ped) {
 	//通常identはjmp先のrule番号@luaだけど
 	if( ped->rdata->last_rpos < jst->rpos || gettoken(ped, -1,-1)->num<0 ) {
 		// EOF消費済みの正当勝利。failに入る前に切りたいけどEOF+があるので判別不能
-		if(jst->curpos<0) {jst->seof=2; rc= -10; goto lb_RTN;}
+		if(jst->curpos<= -10) {jst->seof=2; rc= -10; goto lb_RTN;}
 		//EOF以外。指示がない場合で全文字消費なら完全成功
 		//ぴったりでeofが立ってない場合は確認。endruleなので一歩手前に戻す
 		//大抵はlitがいるのでここに落ちる前にseofが立ちそうだけど
-		int left = sg_sz(jst->instr) - jst->curpos;
+		int left = sg_sz(jst->instr) - curM(jst->curpos);
 		if(jst->seof==0 && left==0) {
 			jst->rpos--;
 			jst->tpos=0;
@@ -1716,7 +1841,6 @@ static jbm_t ped_jmp(ped_t* ped) {
 	// cur初期化とかはworkerの依頼時にこのfuncの外部で処理する
 	jbm_t res={0};
 	jst_t* jst = ped->jst;
-	
 	//mainloop
 	int rc=0;
 	while(1) {
@@ -1735,17 +1859,24 @@ static jbm_t ped_jmp(ped_t* ped) {
 			break;
 		}
 		//通常jmp開始
-		if(jst->curpos < 0) { goto lb_FAIL; }	//EOF consumed. auto fail all
-		int leftsz = sg_sz(jst->instr) - jst->curpos;
+		if(jst->curpos <= -10) { goto lb_FAIL; }	//EOF consumed. auto fail all
+		int leftsz = sg_sz(jst->instr) - curM(jst->curpos);
 		if(jst->seof==0 && leftsz==0) { rc= BLKsz; continue; } //適当に追加
 		//purne
 //dbg(jst->rpos, jst->tpos, tk->data);
 
 //loopだとtpos==1で何度も通ることになるからjmpとかloop以外のフラグが必須
 //vic fail identによるenterはtpos=0にセットして初回を確保する事にした
+
+//loopの検出は二ヶ所。*+系のstxloopとidentのjmploop
+//stxはsucで判定する. jmpはluaでダメ文字リストを作成してclistで弾く
+// jmpはclistがほぼ必須。ここで個別にやるとフラグがぐちゃぐちゃになる
+
+//printf("%d/%d\n", cfail, call);
+//call++;
 		if(jst->tpos== 0 && leftsz>0 ) {
 			jst->tpos=1;
-			int c = (uint8_t)sg_ptr(jst->instr)[jst->curpos];
+			int c = (uint8_t)sg_ptr(jst->instr)[curM(jst->curpos)];
 			int flg = ped->rdata->clists[jst->rpos][c];	// cfilter
 //dbg(c, rc, jst->rpos, jst->tpos, tk->data);
 /*
@@ -1766,7 +1897,7 @@ sleep(1);
 					res_push(ped->jst, TK_LIT, 0);
 				};
 				jst->tpos--;	//sepから一つ前
-				jst->resepos--;	//FAIL先で""が追加される
+				jst->resepos--;	//FAIL先で""が追加される 位置合わせスタンバイ
 				goto lb_FAIL;
 			}
 			else if(flg== -2) {
@@ -1775,7 +1906,7 @@ sleep(1);
 				//するのがいいか>>****否。ここで弾かないとダメ。
 				//本来のloopキックは全部loopck時に弾かなくてはならない
 				//お情けで危ないruleを見逃してるのでダメなcharがきたら
-				//弾かなくてはならない jmploopになってしまう
+				//弾かなくてはならない jmploopは*+のloop検査から外れてる
 				//未消費成功は見てるけどjmp時点ではnestの関係もあってみえない
 				//正常か異常かjmpの時点で判別できない
 				jmp_seterr(ped, "input byte causes infinite loop(dangerous rule)");
@@ -1785,9 +1916,11 @@ sleep(1);
 		}
 		if(jst->tpos==0){jst->tpos=1;}
 		rtoken_t* tk = gettoken(ped, -1,-1);
-		// core start
+		// core start: ident lit any class reg>>bre/ere eof
 		if(tk->ctg[0]=='i') {
 			//ident
+			//identのjmploopはclistの-2が必須になる。ここで逐次やってもいいが
+			//毎回チェックがいるのでひどいovhになる
 			int rpos_dst = tk->num -1;	//luaは1startなのでずれる
 			res_push(ped->jst, TK_IDENT, rpos_dst);	//jmp先numを突っ込むが実際は使わない
 			jst->rpos=rpos_dst;
@@ -1804,10 +1937,10 @@ sleep(1);
 				continue;
 			}
 //dbg( sg_ptr(jst->instr)+jst->curpos, sg_ptr(jst->instr), tk->data, litsz);
-			rc = strncmp( sg_ptr(jst->instr)+jst->curpos, tk->data, litsz);
+			rc = strncmp( sg_ptr(jst->instr)+curM(jst->curpos), tk->data, litsz);
 			if(rc!=0) { goto lb_FAIL; }
 			res_push(ped->jst, TK_LIT, litsz);
-			jst->curpos += litsz;
+			jst->curpos = curM(jst->curpos)+litsz;
 			goto lb_SUC;
 		}
 		if(tk->ctg[0]=='a') {
@@ -1819,7 +1952,7 @@ sleep(1);
 				continue;
 			}
 			res_push(ped->jst, TK_LIT, litsz);	//anyは1文字のlitとして扱う
-			jst->curpos+=litsz;
+			jst->curpos = curM(jst->curpos)+litsz;
 			goto lb_SUC;
 		}
 		if(tk->ctg[0]=='c') {
@@ -1830,40 +1963,47 @@ sleep(1);
 				rc= 1;	//一文字追加要求
 				continue;
 			}
-			char* p = sg_ptr(jst->instr)+jst->curpos;
+			char* p = sg_ptr(jst->instr)+curM(jst->curpos);
 			int cnum = (uint8_t)p[0];
 			rc = tk->data[cnum];
 			if(rc==0) {goto lb_FAIL;}
 			res_push(ped->jst, TK_LIT, litsz);
-			jst->curpos += litsz;
+			jst->curpos = curM(jst->curpos)+litsz;
 			goto lb_SUC;
 		}
-		if(tk->ctg[0]=='r') {
-			//reg
+		if(tk->ctg[1]=='r') {
+			//bre, ere
 			if(jst->seof==0) { rc = -30; continue; }		//full要求 regはlongest
 			ureg_t* rg = getrg(ped, -1, -1);
-			rc = ureg_search_head(rg, sg_ptr(jst->instr)+jst->curpos, leftsz);
+			rc=ureg_search_head(rg,sg_ptr(jst->instr)+curM(jst->curpos),leftsz);
 //dbg(rc, rg->p, rg->sz, jst->curpos, leftsz, jst->seof);sleep(1);
 			if(rc<0){ goto lb_FAIL; }
-			int sz = rg->sz;
-			res_push(ped->jst, TK_LIT, sz);
-			jst->curpos += sz;
+			int litsz = rg->sz;
+			res_push(ped->jst, TK_LIT, litsz);
+			jst->curpos = curM(jst->curpos)+litsz;
 			goto lb_SUC;
 		}
 		if(tk->ctg[0]=='e') {
 			; STOP( tk->pre[0]!=0, "fatal err");
-			//eofは!を削除してるはず
+			//eofは!を削除してるはず ..notEOFは&.をつかうしかないのか
 			// eofは一度ヒットしたら二度と消費しない仕組みが欲しいので-cur
 			// paperにも消費とかかれてるからそっちを採用する
 			if(leftsz==0&&jst->seof==0) { rc = BLKsz; continue; }
 			if(leftsz!=0) { goto lb_FAIL; }
 			//eofヒット
 			res_push(ped->jst, TK_LIT, 0);	//actもあるのでダミーを突っ込む
-			jst->curpos = -jst->curpos -1;
+			jst->curpos = -curM(jst->curpos) -10;
 			//EOF curはresに保存済でrollbackで自動的復帰
 			//バッファとして使うけど0が-+0なので-1で一つずらす
 			goto lb_SUC;
 		}
+		if(tk->ctg[0]=='b') {
+			//bofは全体byteデータとcurposの合成で判定. 消費はない
+			if( jst->bcnt !=0 || jst->curpos!= -1){ goto lb_FAIL; }
+			res_push(ped->jst, TK_LIT, 0);	//actもあるのでダミーを突っ込む
+			jst->curpos = 0;
+			goto lb_SUC;
+		}
 		ERRact(1, "unreachable code", exit(1) );
 lb_FAIL:;
 		rc= -2;
@@ -2063,7 +2203,7 @@ static char* edit_rstop(ped_t* ped, int rnum, char* bin, int binsz, int cur){
 	int pstsz = binsz-cur;
 	if(presz>20){ presz=20; }
 	if(pstsz>20){ pstsz=20; }
-	sg_t* sg = ped->edbuf->sgbuf;
+	sg_t* sg = ped->edbuf->sg1;
 	sg_clear(sg);
 	sg_addf(sg, "ERR: hit '_E': %s: l %d,%d (%d byte)\ninput: %.*s@%.*s"
 		, atk->rname,nlcnt+1, ccnt+1, bcnt+1, presz, bin+cur-presz, pstsz, bin+cur );
@@ -2250,7 +2390,6 @@ static char* edit_srcptr(ped_t* ped, ed_t ed){
 	return p;
 }
 
-
 static char* my_itoa8(int num);
 static char* edit_write_tree(ped_t* ped, ed_t* earr, int earrsz) {
 // こんなの
@@ -2466,6 +2605,7 @@ static int jbm_scwrite(ped_t* ped, int fd, jbm_t* data) {
 			int binsz = data->binsz;
 			sg_add(jst->instr, bin, binsz);
 			jst->rtn = ped_jmp(ped);	//selfのstr_writeはjmpスタートフラグ
+//if(jst->rtn.emsg){puts(jst->rtn.emsg);}
 			//editはreadで適切な情報がきたら動かせば良い
 			//lockはped_parseの上関数で追加削除してる
 		}
@@ -2477,7 +2617,8 @@ static int jbm_scwrite(ped_t* ped, int fd, jbm_t* data) {
 		else if(data->flg==J_CLOSE){ ;}	//ig
 		else {
 			dbg(data->flg);
-			;STOP(1, "fatal"); }	//J_CLOSE, J_IOLOST はselfには送ることがないはず
+			;STOP(1, "fatal");
+		}	//J_CLOSE, J_IOLOST はselfには送ることがないはず
 		goto lb_RTN;
 	}
 
@@ -2492,7 +2633,6 @@ static int jbm_scwrite(ped_t* ped, int fd, jbm_t* data) {
 	//負数のビット表現は環境依存なので-1はb1001かもしれないしb0001かもしれない
 	//aが0x0a 0x01を送って bが0x0a 0x01を受信して1001をb環境でintに直すと
 	// -200になってるかもしれない forkで同一環境なのでたまたま一致してるだけ
-
 	//追加分があるならセット IOとかCLOSEはflgだけ
 	if(data->flg==J_REQSZ){ harr[3] = data->reqsz; }
 	else if(data->flg==J_VICINFO){
@@ -2701,9 +2841,9 @@ static ped_rt ped_rtnjbm(ped_t* ped, jbm_t* jbm, int jidx) {
 	}
 //ERR報告		
 	if(jbm->flg==J_EMSG) {
+		res.emsg = jbm->emsg;
 		ped_reset(ped);
 		res.rc = -10;	//err報告
-		res.emsg = jbm->emsg;
 		goto lb_RTN;
 	}
 	; STOP(1, "unreachable code");
@@ -2806,7 +2946,6 @@ lb_TOP:;
 		goto lb_MERGE;	//全回収
 	}
 	if(msv->seof && sg_sz(msv->instr)==0){ goto lb_MERGE; } //在庫0ならスキップ
-
 	
 //--order
 ;STOP(msv->wait_jidxinfo!=0, "lock break");
@@ -2848,6 +2987,7 @@ lb_MERGE:;
 	sg_clear(sg);
 	rc=0;
 	while(1) {
+
 		if(idq_len(msv->jidx)==0) {break;}
 		int flg = idq_see(msv->strq, 0);
 		if(flg<0) {break;}	// -1で仕事中以外の先頭を集める 出力はoutstrの位置
@@ -2859,6 +2999,7 @@ lb_MERGE:;
 		sg_add(sg, sg_ptr(msv->outstr)+cur, sz);
 	}
 	//返却有り
+
 	if(rc){
 		//gcタイミングは全部捌けた直後だけ
 		if(idq_len(msv->jidx)==0) { sg_clear(msv->outstr); }
@@ -2911,6 +3052,15 @@ int main(){
 /*
  change log
  --
+2021-08-20  Momi-g	<dmy@dmy.dmy>
+
+	* ped.c (ped_jmp): v2.1.0 
+	
+	* ped.c (ped_jmp): add bof, fix clist logic, fix emsg sgbuf>>sg1
+	add regex >> ere + bre. fix doc etc.
+	
+	* ped.c (ped.3): adapt new mandoc script
+
 2021-07-22  Momi-g	<dmy@dmy.dmy>
 
 	* ped.c (ped_jmp): fix clist filter, add -2 etc
