@@ -16,7 +16,7 @@
 */
 
 /*SH_doc
-title=ped section=1
+title=ped section=1 repnl=\040
 @name	ped
 @_brief	sed-style editer using peg
 @synopsis
@@ -222,6 +222,10 @@ int main(int argc, char** argv){
 /*
  change log
  --
+2021-08-20  Momi-g	<dmy@dmy.dmy>
+
+	* ped_main.c (ped.1): adapt new mandoc script
+
 2021-07-05  Momi-g	<dmy@dmy.dmy>
 
 	* ped_main.c (optbox): add -V opt
